// This file was created on 10.31.03 by Andrea Sharp
// to shorten the length of the file MulanDetectorConstruction.cc

// Note that all Material pointers used in MulanDetectorConstruction::buildMaterials()
// must be called according to the names defined in this
// function. Some names may have been changed to make the
// implementation in buildMaterials() more transparent.

#include "MulanDetectorConstruction.hh"
#include "G4Material.hh"
#include "G4Element.hh"

// This function is a private member of class MulanDetectorConstruction
void MulanDetectorConstruction::buildMaterials()
{
  G4double a, z, density;
  G4String name, symbol;
  G4double temperature, pressure;

  // Elements
  G4double aH = 1.00794*g/mole;
  elH = new G4Element("Hydrogen",   symbol="H",  1, aH);
  G4double aC = 12.0107*g/mole;
  elC = new G4Element("Carbon",     symbol="C",  6, aC);
  G4double aN = 14.01*g/mole;
  elN = new G4Element("Nitrogen",   symbol="N",  7, aN);
  G4double aO = 16.00*g/mole;
  elO = new G4Element("Oxygen",     symbol="O",  8, aO);
  G4double aF = 18.9984032*g/mole;
  elF = new G4Element("Flourine",   symbol="F",  9, aF);
  G4double aSi = 28.0855*g/mole;
  elSi = new G4Element("Silicon",   symbol="Si", 14, aSi);
  G4double aS = 32.066*g/mole;
  elS = new G4Element("Sulfur",     symbol="S",  16, aS);
  G4double aCl = 35.4527*g/mole;
  elCl = new G4Element("Chlorine",  symbol="Cl", 17, aCl);
  G4double aCr = 51.9961*g/mole;
  elCr = new G4Element("Chromium",  symbol="Cr", 24, aCr);
  G4double aMn = 54.938049*g/mole;
  elMn = new G4Element("Manganese", symbol="Mn", 25, aMn);
  G4double aFe = 55.845*g/mole;
  elFe = new G4Element("Iron",      symbol="Fe", 26, aFe);
  G4double aCo = 58.933*g/mole;
  elCo = new G4Element("Cobalt",    symbol="Co", 27, aCo);
  G4double aNi = 58.6934*g/mole;
  elNi = new G4Element("Nickel",    symbol="Ni", 28, aNi);
  G4double aW = 183.84*g/mole;
  elW = new G4Element("Tungsten",   symbol="W",  74, aW);
  G4double aSn = 118.710*g/mole;
  elSn = new G4Element("Tin",       symbol="Sn", 50, aSn);
  G4double aAu = 196.967*g/mole;
  elAu = new G4Element("Gold",      symbol="Au", 79, aAu);

  //
  // Bulk Solids
  //

  //Aluminum
  z = 13.0;
  a = 26.98*g/mole;
  density = 2.7*g/cm3;
  Aluminium = new G4Material("Aluminium", z, a, density); 

  //Arnokrome 3 = Magnetizing metal for catcher or target
  density = 7.6*g/cm3;
  Arnokrome3 = new G4Material("Arnokrome3", density, 3, kStateSolid);
  Arnokrome3->AddElement(elCr,.30);
  Arnokrome3->AddElement(elCo,.10);
  Arnokrome3->AddElement(elFe,.60);
  
  //Arnokrome 4 = Magnetizing metal for catcher or target
  density = 7.84*g/cm3;
  Arnokrome4 = new G4Material("Arnokrome4", density, 2, kStateSolid);
  Arnokrome4->AddElement(elCr,.12);
  Arnokrome4->AddElement(elFe,.88);

  //BC404 Scintillator
  density = 1.032*g/cm3;
  BC404 = new G4Material("BC404", density, 2);
  G4double HCRatioBC404 = 1.100;
  G4double HWeightBC404 = HCRatioBC404*aH/(HCRatioBC404*aH +aC);
  BC404->AddElement(elH, HWeightBC404);
  BC404->AddElement(elC, 1-HWeightBC404);

  //BC408 Scintillator
  density = 1.032*g/cm3;
  BC408 = new G4Material("BC408", density, 2);
  G4double HCRatioBC408 = 1.104;
  G4double HWeightBC408 = HCRatioBC408*aH/(HCRatioBC408*aH +aC);
  BC408->AddElement(elH, HWeightBC408);
  BC408->AddElement(elC, 1-HWeightBC408);
  
  //Carbon Fiber Fabric
  density = 1.760*g/cm3;
  CarbonFiber = new G4Material("CarbonFiber", density, 1, kStateSolid);
  CarbonFiber->AddElement(elC, 1.0);  

  //Cobalt
  z = 27.0;
  a = 58.933*g/mole;
  density = 8.90*g/cm3;
  Cobalt = new G4Material("Cobalt", z, a, density);

  //Gold
  z = 79.0;
  a = 196.967*g/mole;
  density = 19.300*g/cm3;
  Gold = new G4Material("Gold", z, a, density);

  //Iron
  z = 26.0;
  a = 55.845*g/mole;
  density = 7.874*g/cm3;
  Iron = new G4Material("Iron", z, a, density);

  //Lead
  z = 82.0;
  a = 207.19*g/mole;
  density = 11.35*g/cm3;
  Lead = new G4Material("Lead", z, a, density);

  //Mylar
  density = 1.39*g/cm3;
  Mylar = new G4Material("Mylar", density, 3);
  Mylar->AddElement(elC, 5);
  Mylar->AddElement(elH, 4);
  Mylar->AddElement(elO, 2);

  //Silver
  z = 47.0;
  a = 107.8682*g/mole;
  density = 10.5*g/cm3;
  Silver = new G4Material("Silver", z, a, density);
  
  //Stainless Steel
  density = 7.9*g/cm3;
  SSteel = new G4Material("SSteel", density, 5, kStateSolid);
  SSteel->AddElement(elCr,.19);
  SSteel->AddElement(elFe,.6875);
  SSteel->AddElement(elMn,.02);
  SSteel->AddElement(elNi,.0925);
  SSteel->AddElement(elSi,.01);
  
  //Sulfur
  z = 16.0;
  a = 32.006*g/mole;
  //   density = 2.06860*g/cm3;
  density = 1.58*g/cm3;  //this is target made and spec'd by Bernhard
  Sulfur = new G4Material("Sulfur", z, a, density);
  
  //Tin
  z = 50.0;
  a = 118.710*g/mole;
  density = 7.310*g/cm3;
  Tin = new G4Material("Tin", z, a, density);
  
  //Tungsten
  z = 74.0;
  a = 183.84*g/mole;
  density = 19.250*g/cm3;
  Tungsten = new G4Material("Tungsten", z, a, density);
  
  //
  // Gases
  //

  //Air
  density = 1.2931*mg/cm3;
  Air = new G4Material("Air", density, 2);
  Air->AddElement(elN, .7);
  Air->AddElement(elO, .3);

  //Argon gas
  z = 18.0;
  a = 39.948*g/mole;
  density     = 1.782*mg/cm3;
  temperature = 293.15*kelvin;
  pressure    = 1*atmosphere;
  ArgonGas = new G4Material("ArgonGas", z, a, density, kStateGas,
			    temperature, pressure);
  
  //Freon gas
  density     = 5.418*mg/cm3;
  temperature = 293.15*kelvin;
  pressure    = 1*atmosphere;
  FreonGas = new G4Material("FreonGas", density, 3, kStateGas,
			    temperature, pressure);
  FreonGas->AddElement(elC, 1);
  FreonGas->AddElement(elCl, 2);
  FreonGas->AddElement(elF, 2);
  
  //Helium gas
  z = 2.0;
  a = 4.002602*g/mole;
  density     = .1786 *mg/cm3;
  temperature = 293.15*kelvin;  //room temperature, not STP
  pressure    = 1*atmosphere;
  HeliumGas = new G4Material("HeliumGas", z, a, density, kStateGas,
			     temperature, pressure);
  
  //Methane gas
  density     = 0.6657*mg/cm3;  //look this up
  temperature = 293.15*kelvin;
  pressure    = 1*atmosphere;
  MethaneGas = new G4Material("MethaneGas", density, 2, kStateGas,
			      temperature, pressure);
  MethaneGas->AddElement(elC, 1);
  MethaneGas->AddElement(elH, 4);
  
  //Vacuum (Galactic)
  density     = 1.e-25*g/cm3;
  pressure    = 1.e-25*atmosphere;
  temperature = pressure/(density*k_Boltzmann);
  
  //Vacuum (beam)
  //   density     = 1.e-7*mg/cm3;
  //   pressure    = 2.e-2*atmosphere;
  //   temperature = pressure/(density*k_Boltzmann);
  Vacuum = new G4Material("Vacuum", density, 1, kStateGas,
			  temperature, pressure);
  Vacuum->AddMaterial(Air, 1.);
  
  //Wire Chamber Gas Mixture - methane (CH4) plus helium (He)
  density     = 0.6657*mg/cm3;  //look this up
  temperature = 293.15*kelvin;
  pressure    = 1*atmosphere;
  WireChamberGas = new G4Material("WireChamberGas", density, 2,
				  kStateGas, temperature, pressure);
  WireChamberGas->AddMaterial(HeliumGas, .5);
  WireChamberGas->AddMaterial(MethaneGas, .5);
  
  //Xenon gas
  z = 54.0;
  a = 131.29*g/mole;
  density     = 5.858*mg/cm3;
  temperature = 293.15*kelvin;
  pressure    = 1*atmosphere;
  XenonGas = new G4Material("XenonGas", z, a, density, kStateGas,
			    temperature, pressure);
}
