// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: MulanPrimaryGeneratorAction.cc,v 1.3 2004/01/30 17:21:26 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#include "MulanPrimaryGeneratorAction.hh"
#include "MulanDetectorConstruction.hh"

#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "globals.hh"
#include "Randomize.hh"
#include <math.h>
#include <stdlib.h>

#ifndef MUONS_CREATED_AT_TARGET   // this is a control switch for creating
#define MUONS_CREATED_AT_TARGET 1 // the muons at the target with no energy
#endif                            // i.e. they decay right away.
                         // change the 0 to 1 if you want to run in this mode.

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanPrimaryGeneratorAction::MulanPrimaryGeneratorAction(MulanDetectorConstruction* myDC,char* inputFile):myDetector(myDC)
{
  G4cout<<"MulanPrimaryGeneratorAction instantiating... ";
  G4int n_particle = 1;
  particleGun = new G4ParticleGun(n_particle);

  // define default (dummy) particle
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  G4ParticleDefinition* particle = particleTable->FindParticle("mu+");
  
  particleGun->SetParticleDefinition(particle);
  particleGun->SetParticleMomentumDirection(G4ThreeVector(0.,0.,1.));
  //  particleGun->SetParticleEnergy(4.0*MeV);
  //  particleGun->SetParticleEnergy(3.495*MeV);
  //  particleGun->SetParticleEnergy(5.646*MeV);
  particleGun->SetParticleEnergy(3.6692*MeV);
  //  particleGun->SetParticleEnergy(24.70*MeV);
  particleGun->SetParticlePolarization(G4ThreeVector(0.,0.,-1.));
  position = -1.0*meter;
  particleGun->SetParticlePosition(G4ThreeVector(0,0,position));
  
  if(inputFile) fInputFile = new ifstream(inputFile,ios::in);
  else fInputFile = NULL;

  G4cout << "done"<<G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanPrimaryGeneratorAction::~MulanPrimaryGeneratorAction()
{
  delete particleGun;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{ 
  G4double r;
  G4double x;
  G4double y;
  G4double xprime;
  G4double yprime;
  G4double momentum;
  G4double discard1;
  G4double discard2;
  G4double energy;
  G4double xMom;
  G4double yMom;
  G4double zMom;

  if(fInputFile)
  {
    *fInputFile >> x >> xprime >> y >> yprime >> momentum>>discard1>>discard2;

    // transform into Mulan coordinates 
    position = -150*cm;
    // Obtain momentum components
    x = x*cm;  //if the particles provided are originating at the
    y = y*cm;  //entrance to our beam pipe
    xprime = xprime/1000; //divide by 1000 to get to radians
    yprime = yprime/1000;
    x = x + position*xprime; //correct for particle origin
    y = y + position*yprime;    
    momentum = momentum*MeV;

    G4double mass = 105.65836*MeV;
    energy = sqrt(momentum*momentum + mass*mass) - mass;
    xMom = xprime;
    yMom = yprime;
    zMom = sqrt(1-xMom*xMom-yMom*yMom);
    particleGun->SetParticleMomentumDirection(G4ThreeVector(xMom,yMom,zMom));
    particleGun->SetParticleEnergy(energy);

  }

  else  // options if no input file specified
  // execute the following if MUONS_CREATED_AT_TARGET == 1
#if MUONS_CREATED_AT_TARGET
  {
    position = 0.0*meter;
    x = 0;
    y = 0;

    do {
    x = (1*G4UniformRand()-0.5)*mm;  //these 2 statements make the beam profile
    y = (1*G4UniformRand()-0.5)*mm;  //into a uniform 1 mm radius circle
    r = sqrt(x*x + y*y);
    } while (r > 0.5*mm);
  }
  //  energy = 1000.0 * eV;  // put them at the target with no energy
  //  zMom = 1.;          // and wait for them to decay
  energy = 0.001 * eV;  // put them at the target with no energy
  zMom = 0.;          // and wait for them to decay
  xMom = 0.;
  yMom = 0.;
  // override the previous definitions
  particleGun->SetParticleMomentumDirection(G4ThreeVector(xMom,yMom,zMom));
  particleGun->SetParticleEnergy(energy);

#else
  // second option; normal running mode 
  // if MUONS_CREATED_AT_TARGET == 0
  {
    position = -1.0*meter;
    x = 0;
    y = 0;

    do {
    x = (1*G4UniformRand()-0.5)*mm;  //these 2 statements make the beam profile
    y = (1*G4UniformRand()-0.5)*mm;  //into a uniform 1 mm radius circle
    r = sqrt(x*x + y*y);
    } while (r > 0.5*mm);
  }
#endif

    // these statements required no matter what
  particleGun->SetParticlePolarization(G4ThreeVector(0.,0.,-1.));
  particleGun->SetParticlePosition(G4ThreeVector(x,y,position));  
  particleGun->GeneratePrimaryVertex(anEvent);

}
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

