#include "MulanDetectorConstruction.hh"
#include "MulanDetectorConstruction.ecc"
#include "MulanDetectorMessenger.hh"
#include "MulanMagneticField.hh"
#include "MulanDetectorSD.hh"

#include "G4Box.hh"
#include "G4Trd.hh"
#include "G4Trap.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4UnionSolid.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVParameterised.hh"
#include "G4SDManager.hh"
#include "G4Polyhedra.hh"
#include "G4Polycone.hh"
#include "G4UserLimits.hh"
#include "G4Assembly.hh"
#include "G4AssemblyCreator.hh"
#include "G4TransportationManager.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4ios.hh"

#ifndef TARGET_IS_AK3   // this is a control switch for setting the
#define TARGET_IS_AK3 1 // target material to Arnokrome 3
#endif                  // as it stands, this config is the DEFAULT.
                        // change the 1 to 0 and recompile to change
                        // the target config to sulfur or silver disk
                        // and a ring of magnets.

#ifndef WIRE_CHAMBER_IN_USE   // this is a control switch to take the wire
#define WIRE_CHAMBER_IN_USE 0 // chamber in and out of the geometry. default
#endif                        // config is OUT. change 0 to 1 and recompile
                              // to put it back in.

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorConstruction::MulanDetectorConstruction()
      :solidWorld(NULL),  logicWorld(NULL),  physiWorld(NULL),
      solidTarget(NULL), logicTarget(NULL), physiTarget(NULL),
      TargetMater(NULL), fpMagField(NULL),
      WorldRadius(0.),  fTargetLength(0.), fHeBagLength(0.)

{
  //  G4cout<<"MulanDetectorConstruction::MulanDetectorConstruction begun."<<G4endl;
   fpMagField = new MulanMagneticField();
   detectorMessenger = new MulanDetectorMessenger(this);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorConstruction::~MulanDetectorConstruction()
{
   delete fpMagField;
   delete detectorMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4VPhysicalVolume* MulanDetectorConstruction::Construct()
{
   buildMaterials();

   //--------- Sizes of the principal geometrical components (solids)  -------

   //   useful in setting all matter to vacuum
   //   Mylar = Vacuum;
   //   HeliumGas = Vacuum;
   //   Air = Vacuum;

   hexPsi = pi/6.;  hexTheta = pi/3.;
   pentPsi = pi/5.; pentTheta = 3.*pi/10.;

   //Tile Dimensions
   baseTile = 15. * cm;                 //length of tile base
   thicknessInTile = .3 * cm;           //thickness of inner tile
   thicknessOutTile = thicknessInTile;  //thickness of outer tile
   spaceTile = 0.1 * cm;                //spacing between inner and outer tiles
   InMinusOut = thicknessInTile;        //Diff between tiles along LG edges
   inTileMater = BC404;
   outTileMater = inTileMater;
   spacingTile = 1*cm;                  //spacing between adjacent tiles


   //LightGuide Dimensions
   lengthLG = 20.32*cm;           //height or length of LG
   baseLG = baseTile;             //length of LG where scint & LG meet
   topLG = 2.76788*cm;            //size of square where PMT attaches
   thicknessLG = .3175*cm;        //thickness of LG
   offsetLG = 2.2225*cm;          //distance from top of LG to edge of bottom of LG
   LGMater = Mylar;               //for now

   //Super Hexagon and Pentagon Dimensions
   //  widthSH = .5*(thicknessInTile+topLG)+.1 * cm; //space between tile and Super hex/pentagon
   widthSH = .635*cm;
   depthSH = 40 * cm;                   //length of Super hex/pentagon house
   baseSH = (baseTile + spacingTile/sin(hexTheta) + 2*widthSH/tan(hexTheta)); //length of SH base
   SHMater = Air;                       //super hex/pentagon house material
   radiusSH = tan(hexTheta)*baseSH/2.;  //radius of circle inscribing SH
   radiusSP = tan(pentTheta)*baseSH/2.; //radius of circle inscribing SP

   //Constants of Icosahedron
   G4double X = .525731112119133606;    // distance from center to
   G4double Z = .850650808352039932;    // apex of triangles assuming unit length
   //  G4double R = sqrt(1./4. - X*X/3.)/X;  //Radius inscribing unit super triangle
   domeRadius = baseSH*sqrt(42.+18.*sqrt(5.))/4.;
   domeSPRadius = sqrt(domeRadius*domeRadius + radiusSH*radiusSH - radiusSP*radiusSP);
   //   G4cout << "dome SPRadius = beamToCenter = " << domeSPRadius << "\n";

   // Actual House Dimensions
   wallThickness = 0.762*mm;
   wallHeight = depthSH - 2*cm;
   wallLengthSH = baseSH - 2*tan(hexPsi)*wallThickness;
   wallLengthSP = baseSH - 2*tan(pentPsi)*wallThickness;
   wallMater = Air;

   // Beam Line and Helium Bag Parameters
   lengthBeamLine = 150*cm;
   radiusBeamLine = 10*cm;
   beamToCenter = domeSPRadius;
   BeamLineMater = Vacuum;
   lengthBeamWindow = 100*micrometer;  // for normal operations
   BeamWindowMater = Mylar;

   //   lengthBeamWindow = 1.58*mm;   //  for testing NIM data
   //BeamWindowMater = Aluminium;
 
   G4Material* bagMater = HeliumGas;
   G4double thicknessBag = 0.0254*micrometer;  //full thickness of bag itself (mm)
   G4double HeBagLength = beamToCenter;


   radiusBeamLine = 12.5*cm;
   G4double TBoxX = radiusBeamLine;    //transition box half-dimensions
   TBoxX = 5.17*2.54*cm;               //for a 5.17 inch WC

   TBoxX = 81.14/2*mm;          //as per PSI drawings
   G4double TBoxY = 81.27/2*mm; //as per PSI drawings
   G4double TBoxZ = 21./2*mm;   //as per PSI drawings
 

   //--------- Set Visualization Attributes -------------------------------------
   whiteSolid   = new G4VisAttributes(G4Colour(1,1,1));
   blackSolid   = new G4VisAttributes(G4Colour(0,0,0));
   redSolid     = new G4VisAttributes(G4Colour(1,0,0));
   greenSolid   = new G4VisAttributes(G4Colour(0,1,0));
   blueSolid    = new G4VisAttributes(G4Colour(0,0,1));
   magentaSolid = new G4VisAttributes(G4Colour(1,0,1));
   cyanSolid    = new G4VisAttributes(G4Colour(0,1,1));
   yellowSolid  = new G4VisAttributes(G4Colour(1,1,0));
   whiteFrame   = new G4VisAttributes(G4Colour(1,1,1));
   redFrame     = new G4VisAttributes(G4Colour(1,0,0));
   greenFrame   = new G4VisAttributes(G4Colour(0,1,0));
   blueFrame    = new G4VisAttributes(G4Colour(0,0,1));
   magentaFrame = new G4VisAttributes(G4Colour(1,0,1));
   cyanFrame    = new G4VisAttributes(G4Colour(0,1,1));
   yellowFrame  = new G4VisAttributes(G4Colour(1,1,0));
   offAtt       = new G4VisAttributes();
   TargetVisAtt = new G4VisAttributes();

   whiteSolid->SetForceSolid(true);
   blackSolid->SetForceSolid(true);
   redSolid->SetForceSolid(true);
   greenSolid->SetForceSolid(true);
   blueSolid->SetForceSolid(true);
   magentaSolid->SetForceSolid(true);
   cyanSolid->SetForceSolid(true);
   yellowSolid->SetForceSolid(true);
   whiteFrame->SetForceWireframe(true);
   redFrame->SetForceWireframe(true);
   greenFrame->SetForceWireframe(true);
   blueFrame->SetForceWireframe(true);
   magentaFrame->SetForceWireframe(true);
   cyanFrame->SetForceWireframe(true);
   yellowFrame->SetForceWireframe(true);
   offAtt->SetVisibility(false);

   //------------------------------------------------
   // Create Sensitive Detector Manager
   //------------------------------------------------

   G4SDManager* SDman = G4SDManager::GetSDMpointer();

   G4String SDname = "Mulan/SD";
   MulanDetectorSD* aSD = new MulanDetectorSD( SDname );
   SDman->AddNewDetector( aSD );


   //--------- Definitions of Solids, Logical Volumes, Physical Volumes ---------

   // syntax for solid, logic, and physical volumes:

   //G4SolidClass* solidVolume = new G4SolidClass("name",dimensions parameters);
   //G4LogicalVolume* logicVolume = new G4LogicalVolume(solidVolume,
   //                    material it contains, "logical volume name", 0,0,0 [unknown meaning of 3 zeros]);
   //G4PVPhysicalVolume* physiVolume = new G4PVPlacement(rotation angle, name of position vector,"logical volume 
   //                    name", "physical volume name", logical mother volume to place it in, Boolean
   //                    operations, copy number: usu. = 0);

   //utilized Solid Classes are G4Box("name", x dim, Y dim, Z dim); dim means half-lengths here
   //                       and G4Tubs("name", inner radius, outer radius, half length, starting angle,
   //                       spanning angle);
   // and others like G4Polyhedra.
 



   //------------------------------
   // World
   //------------------------------
   G4Material* worldMater = Air;
   WorldRadius = 3*(domeRadius+depthSH);
   solidWorld  = new G4Sphere("World",0.,WorldRadius,0.,twopi,0.,pi);
   logicWorld  = new G4LogicalVolume( solidWorld, worldMater, "WorldLV", 0, 0, 0);
   physiWorld  = new G4PVPlacement(0,G4ThreeVector(),"WorldPV",logicWorld,0,false,0);

   logicWorld->SetSensitiveDetector(aSD);  
   logicWorld->SetVisAttributes(offAtt); 

   //------------------------------
   //Sphere enclosing soccer ball
   //------------------------------
   G4Material* sphereMater = Air;
   G4RotationMatrix* sphereRM = new G4RotationMatrix();
   sphereRM->rotateX(0.);
   sphereRM->rotateZ(pi/10.);
   sphereRM->rotateY(-atan(X/Z));
   G4double rMinSphere = domeRadius*.98; //changed back, 6/6/03
   G4double rMaxSphere = (domeRadius+depthSH)*1.1;
   solidSphere = new G4Sphere("Sphere",rMinSphere,rMaxSphere,0.,twopi,0.,pi);
   logicSphere = new G4LogicalVolume(solidSphere, sphereMater,"SphereLV",0,0,0);
   physiSphere = new G4PVPlacement(sphereRM,G4ThreeVector(),"SpherePV",logicSphere,physiWorld,false,0);
   logicSphere->SetSensitiveDetector(aSD);  //Set as sensitive
   logicSphere->SetVisAttributes(blueFrame);
   logicSphere->SetVisAttributes(offAtt);  

   //-----------------------------
   // Super Hexagons and Tiles
   //-----------------------------
   G4double rin[2]={0,0};
   G4double rout[2]={radiusSH,radiusSH};
   G4double zPos[2]={0,depthSH};

   solidSH = new G4Polyhedra("SuperHex",hexPsi,twopi,6,2,zPos,rin,rout);

   setHexTileParameters(0);
   solidInTile = new G4Trap("inTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setHexTileParameters(1);
   solidOutTile = new G4Trap("outTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   G4Box* solidWallSH = new G4Box("wallSH",wallThickness/2,wallLengthSH/2,wallHeight/2);
   G4LogicalVolume* logicWallSH[20][6];
   G4VPhysicalVolume* physiWallSH[20][6];

   setHexLGParameters(0,0);
   G4Trap* InHexLG1 = new G4Trap("InHexLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setHexLGParameters(0,1);
   G4Trap* InHexLG2 = new G4Trap("InHexLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   G4UnionSolid* InHexLG = new G4UnionSolid("InHexLG",InHexLG1,InHexLG2,hexLGRotate(0),hexLGTranslate(0));

   setHexLGParameters(1,0);
   G4Trap* OutHexLG1 = new G4Trap("OutHexLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setHexLGParameters(1,1);
   G4Trap* OutHexLG2 = new G4Trap("OutHexLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   G4UnionSolid* OutHexLG = new G4UnionSolid("OutHexLG",OutHexLG1,OutHexLG2,hexLGRotate(1),hexLGTranslate(1));

   char nameSH[50];
   for(G4int i = 0; i<20; i++)
   {
      //super hex
      logicSH[i] = new G4LogicalVolume(solidSH,SHMater,"SuperHexLV",0,0,0);
      sprintf(nameSH,"SuperHexPV%d",i);
      physiSH[i] = new G4PVPlacement(hexRM(i),hexPosition(i),nameSH,logicSH[i],physiSphere,true,i);


      for(G4int j = 0; j<6; j++)
      {
         //walls
         logicWallSH[i][j] = new G4LogicalVolume(solidWallSH,wallMater,"hexWallLV",0,0,0);
         physiWallSH[i][j] = new G4PVPlacement(hexWallRM(i,j),hexWallPosition(i,j),"hexWall",logicWallSH[i][j],physiSH[i],true,j);

         //tile elements
         logicInTile[i][j] = new G4LogicalVolume(solidInTile,inTileMater,"inTileElementLV",0,0,0);
         logicOutTile[i][j] = new G4LogicalVolume(solidOutTile,outTileMater,"outTileElementLV",0,0,0);
         logicSHInLG[i][j] = new G4LogicalVolume(InHexLG,LGMater,"inSH_LG_LV",0,0,0);
         logicSHOutLG[i][j] = new G4LogicalVolume(OutHexLG,LGMater,"outSH_LG_LV",0,0,0);

         physiInTile[i][j] = new G4PVPlacement(hexTileRM(i,j),hexTilePosition(i,j),"InnerTileElement",
					       logicInTile[i][j],physiSH[i],true,j);
         physiOutTile[i][j] = new G4PVPlacement(hexTileRM(i,j+6),hexTilePosition(i,j+6),"OuterTileElement",
						logicOutTile[i][j],physiSH[i],true,j);
         physiSHInLG[i][j] = new G4PVPlacement(hexLGRM(i,j),hexLGPosition(i,j),"InnerHexLG",
					       logicSHInLG[i][j],physiSH[i],true,j);
         physiSHOutLG[i][j] = new G4PVPlacement(hexLGRM(i,j+6),hexLGPosition(i,j+6),"OuterHexLG",
						logicSHOutLG[i][j],physiSH[i],true,j);

         //Set Sensitive Detectors
         logicInTile[i][j]->SetSensitiveDetector(aSD);
         logicOutTile[i][j]->SetSensitiveDetector(aSD);
         logicSHInLG[i][j]->SetSensitiveDetector(aSD);
         logicSHOutLG[i][j]->SetSensitiveDetector(aSD);
         logicWallSH[i][j]->SetSensitiveDetector(aSD);

         //Set Vis Properties
	 logicInTile[i][j] ->SetVisAttributes(magentaSolid);
         logicOutTile[i][j]->SetVisAttributes(redSolid); 
         logicSHInLG[i][j]->SetVisAttributes(whiteFrame);  
	 logicSHOutLG[i][j]->SetVisAttributes(whiteFrame); 
	 //	 logicInTile[i][j] ->SetVisAttributes(offAtt);  
	 logicOutTile[i][j]->SetVisAttributes(offAtt);  
	 logicSHInLG[i][j]->SetVisAttributes(offAtt);
	 logicSHOutLG[i][j]->SetVisAttributes(offAtt);
	 logicWallSH[i][j]->SetVisAttributes(offAtt);
      }
      logicSH[i]->SetSensitiveDetector( aSD);   //sets as sensitive detector
      logicSH[i]->SetVisAttributes(offAtt);     // this is really the hex house
   }

   //-----------------------------
   // Super Pentagons and Tiles
   //-----------------------------
   rout[0]=radiusSP;
   rout[1]=radiusSP;
   zPos[0]= 0;
   zPos[1]= depthSH;

   solidSP = new G4Polyhedra("SuperPent",0,twopi,5,2,zPos,rin,rout);

   G4Box* solidWallSP = new G4Box("wallSP",wallThickness/2,wallLengthSP/2,wallHeight/2);
   G4LogicalVolume* logicWallSP[12][5];
   G4VPhysicalVolume* physiWallSP[12][5];

   setPentTileParameters(0);
   solidSPInTile = new G4Trap("inSPTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   setPentTileParameters(1);
   solidSPOutTile = new G4Trap("outSPTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   setPentLGParameters(0,0);
   G4Trap* InPentLG1 = new G4Trap("InPentLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setPentLGParameters(0,1);
   G4Trap* InPentLG2 = new G4Trap("InPentLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   G4UnionSolid* InPentLG = new G4UnionSolid("InPentLG",InPentLG1,InPentLG2,pentLGRotate(0),pentLGTranslate(0));

   setPentLGParameters(1,0);
   G4Trap* OutPentLG1 = new G4Trap("OutPentLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setPentLGParameters(1,1);
   G4Trap* OutPentLG2 = new G4Trap("OutPentLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   G4UnionSolid* OutPentLG = new G4UnionSolid("OutPentLG",OutPentLG1,OutPentLG2,pentLGRotate(1),pentLGTranslate(1));
   for(i = 1; i<11; i++)
   {
      //super Pentagons
      logicSP[i] = new G4LogicalVolume(solidSP,SHMater,"SuperPentLV",0,0,0);
      physiSP[i] = new G4PVPlacement(pentRM(i),pentPosition(i),"SuperPentPV",logicSP[i],physiSphere,true,i);                // identification
      for(G4int j = 0; j<5; j++)
      {
         //walls
         logicWallSP[i][j] = new G4LogicalVolume(solidWallSP,wallMater,"pentWallLV",0,0,0);
         physiWallSP[i][j] = new G4PVPlacement(pentWallRM(i,j),pentWallPosition(i,j),"pentWall",logicWallSP[i][j],physiSP[i],true,j);

         //Pentagon Tile elements
         logicSPInTile[i][j] = new G4LogicalVolume(solidSPInTile,inTileMater,"inSPTileElementLV",0,0,0);
         logicSPOutTile[i][j] = new G4LogicalVolume(solidSPOutTile,outTileMater,"outSPTileElementLV",0,0,0);
         logicSPInLG[i][j] = new G4LogicalVolume(InPentLG,LGMater,"inSP_LG_LV",0,0,0);
         logicSPOutLG[i][j] = new G4LogicalVolume(OutPentLG,LGMater,"outSP_LG_LV",0,0,0);

         physiSPInTile[i][j] = new G4PVPlacement(pentTileRM(i,j),pentTilePosition(i,j),"InnerSPTileElement",logicSPInTile[i][j],physiSP[i],true,j);
         physiSPOutTile[i][j] = new G4PVPlacement(pentTileRM(i,j+5),pentTilePosition(i,j+5),"OuterSPTileElement",logicSPOutTile[i][j],physiSP[i],true,j);

         physiSPInLG[i][j] = new G4PVPlacement(pentLGRM(i,j),pentLGPosition(i,j),"InnerPentLG",logicSPInLG[i][j],physiSP[i],true,j);
         physiSPOutLG[i][j] = new G4PVPlacement(pentLGRM(i,j+5),pentLGPosition(i,j+5),"OuterPentLG",logicSPOutLG[i][j],physiSP[i],true,j);

         // Set Sensitive Detectors
         logicSPInTile[i][j]->SetSensitiveDetector(aSD);
         logicSPOutTile[i][j]->SetSensitiveDetector(aSD);
         logicSPInLG[i][j]->SetSensitiveDetector(aSD);
         logicSPOutLG[i][j]->SetSensitiveDetector(aSD);
         logicWallSP[i][j]->SetSensitiveDetector(aSD);


         // Set Vis Properties
	 logicSPInTile[i][j] ->SetVisAttributes(cyanSolid);
         logicSPOutTile[i][j]->SetVisAttributes(blueSolid); 
         logicSPInLG[i][j]->SetVisAttributes(whiteFrame); 
         logicSPOutLG[i][j]->SetVisAttributes(whiteFrame);   
	 //	 logicSPInTile[i][j] ->SetVisAttributes(offAtt);
	 logicSPOutTile[i][j]->SetVisAttributes(offAtt);
         logicSPInLG[i][j]->SetVisAttributes(offAtt);
	 logicSPOutLG[i][j]->SetVisAttributes(offAtt);
	 logicWallSP[i][j]->SetVisAttributes(offAtt);
      }

      logicSP[i]->SetSensitiveDetector(aSD);   //sets as sensitive detector
      logicSP[i]->SetVisAttributes(offAtt);     // this is really the pent house 
   }


  // Beamline elements, working upstream from the target // ***************

#if TARGET_IS_AK3
   //------------------------------
   // Arnokrome 3 Square Target
   //------------------------------
   targetSize = 8.0*2.54*cm;     // setting the side length (inches)
   targetThick = 240*micrometer; // and thickness
   TargetMater = Arnokrome3;     // and material to AK3 parameters
   G4ThreeVector positionTarget = G4ThreeVector(0.,0.,0.);
   solidAK3Target = new G4Box("TargetSolid",targetSize/2.,targetSize/2.,targetThick/2.);
   logicTarget = new G4LogicalVolume(solidAK3Target,TargetMater,"TargetLV",0,0,0);
   physiTarget = new G4PVPlacement(0,positionTarget,"TargetPV",logicTarget,physiWorld,false,0);

   logicTarget->SetSensitiveDetector(aSD);
   logicTarget->SetVisAttributes(greenSolid);
   //   logicTarget->SetVisAttributes(offAtt);

   //Set Magnetic Field in Target Region
   fMagField = 9000 * gauss;
   //   fMagField = 0.0 * gauss;
   G4UniformMagField* magField = new G4UniformMagField(G4ThreeVector(0.,fMagField,0));
   G4FieldManager* magManager  = new G4FieldManager(magField);
   magManager->CreateChordFinder(magField);
   logicTarget->SetFieldManager(magManager,true);
#else
   //------------------------------
   // Sulfur or Silver Disk Target
   //------------------------------
   targetSize = 7.5*cm;   // setting the radius
   targetThick = 2*mm;    // and thickness
   TargetMater = Sulfur;  // and material of the disk
   G4ThreeVector positionTarget = G4ThreeVector(0.,0.,0.);
   solidTarget = new G4Tubs("TargetSolid",0.,targetSize,targetThick/2.,0.,twopi);
   logicTarget = new G4LogicalVolume(solidTarget,TargetMater,"TargetLV",0,0,0);
   physiTarget = new G4PVPlacement(0,positionTarget,"TargetPV",logicTarget,physiWorld,false,0);
   if(TargetMater == Silver) { TargetVisAtt->SetColor(.230,.232,.250); }
   else { TargetVisAtt->SetColor(1,0,1); }
   logicTarget->SetSensitiveDetector(aSD);
   logicTarget->SetVisAttributes(greenSolid);
   //   logicTarget->SetVisAttributes(offAtt);

   //Set Magnetic Field in Target Region
   fMagField = 75 * gauss; // look this up
   G4UniformMagField* magField = new G4UniformMagField(G4ThreeVector(0.,fMagField,0));
   G4FieldManager* magManager  = new G4FieldManager(magField);
   magManager->CreateChordFinder(magField);
   logicTarget->SetFieldManager(magManager,true);

   //------------------------------
   // Ring of magnets around the Silver/Sulfur Target only
   //------------------------------
   G4double magnetRingThick = 1*cm;     // Thickness of plastic ring around target
   G4double magnetRingRadius = 5*cm;
   G4Material* MagnetRingMater = Iron;    // to catch muons which traverse magnet ring

   G4ThreeVector positionMagnetRing = G4ThreeVector(0.,0.,0);
   G4Tubs* solidMagnetRing = new
     G4Tubs("MagnetRingSolid",targetSize,targetSize+magnetRingRadius,magnetRingThick/2.,0,twopi);
   G4LogicalVolume* logicMagnetRing = new
     G4LogicalVolume(solidMagnetRing,MagnetRingMater,"MagnetRingLV",0,0,0);
   G4VPhysicalVolume* physiMagnetRing = new 
     G4PVPlacement(0,positionMagnetRing,"MagnetRingPV",logicMagnetRing,physiWorld,false,0);
   logicMagnetRing->SetVisAttributes(whiteSolid);
   //   logicMagnetRing->SetVisAttributes(offAtt);
   logicMagnetRing->SetSensitiveDetector(aSD);
   logicMagnetRing->SetFieldManager(magManager,true);

   //--------------------------------------------------
   // Catcher Ring Around the Silver/Sulfur Target only 
   //--------------------------------------------------
   G4double catcherRingRadius = 15*cm;
   G4double catcherRingThick = 0.1*2.54*cm; // 0.1 inch
   G4Material* CatcherRingMater = Arnokrome3;

   G4ThreeVector positionCatcherRing = G4ThreeVector(0.,0.,0);
   G4Tubs* solidCatcherRing = new
     G4Tubs("CatcherRingSolid",targetSize+magnetRingRadius,targetSize+magnetRingRadius+catcherRingRadius, catcherRingThick/2 ,0,twopi);
   G4LogicalVolume* logicCatcherRing = new
     G4LogicalVolume(solidCatcherRing, CatcherRingMater,"CatcherRingLV",0,0,0);
   G4VPhysicalVolume* physiCatcherRing = new
     G4PVPlacement(0,positionCatcherRing,"CatcherRingPV",logicCatcherRing,physiWorld,false,0);
   logicCatcherRing->SetVisAttributes(yellowSolid);
   //   logicCatcherRing->SetVisAttributes(offAtt);
   logicCatcherRing->SetSensitiveDetector(aSD);

#endif

   //------------------------------
   // T0 (just upstream of and touching the target now)
   //------------------------------
   G4double thicknessT0 = 500.0*micrometer; //does not work if = 0 !
   G4Material* t0Mater = BC404; 
   //   G4Material* t0Mater = Vacuum; 

#ifdef magnetRingThick
   G4ThreeVector positionT0 = G4ThreeVector(0,0,-targetThick/2 -magnetRingThick/2 -thicknessT0/2);
#else
   G4ThreeVector positionT0 = G4ThreeVector(0,0,-targetThick/2 - thicknessT0/2);
#endif

   G4double T0sidelength_full = 7.5*2.54*cm;
   G4Box* solidT0 = new G4Box("T0",T0sidelength_full/2,T0sidelength_full/2,thicknessT0/2);

   //   G4Tubs* solidT0 = new
   //     G4Tubs("T0",0.,radiusBeamLine-1*cm,thicknessT0/2.,0.,twopi);

   G4LogicalVolume* logicT0 = new
     G4LogicalVolume(solidT0,t0Mater,"T0LV",0,0,0);

   G4VPhysicalVolume* physiT0 = new
     G4PVPlacement(0,positionT0,"T0",logicT0,physiWorld,false,0);

   logicT0->SetSensitiveDetector(aSD);
   logicT0->SetVisAttributes(yellowSolid);
   //   logicT0->SetVisAttributes(offAtt);

   //-----------------------------
   // Beam Window Holder (red on drawing, touching T0) 
   //----------------------------
   G4double innerWH = 7.5*cm;
   G4double lengthWH = 0.5 * cm;
   G4Tubs *solidWindowHolder = new   // define shape
     G4Tubs("WindowHolder",innerWH,innerWH+0.4*cm,lengthWH/2,0.,twopi);

   G4LogicalVolume *logicWindowHolder = new   // make 1 instance of it
     G4LogicalVolume(solidWindowHolder,Mylar,"WindowHolderLV",0,0,0);
   //   G4LogicalVolume *logicWindowHolder = new   // make 1 instance of it
   //     G4LogicalVolume(solidWindowHolder,Vacuum,"WindowHolderLV",0,0,0);

   G4ThreeVector positionWindowHolder =
     G4ThreeVector(0,0,-targetThick/2 - thicknessT0 - lengthWH/2);

   G4VPhysicalVolume *physiWindowHolder = new
     G4PVPlacement(0,positionWindowHolder,"WindowHolder",logicWindowHolder,physiWorld,false,0);

   logicWindowHolder->SetVisAttributes(redSolid);
   logicWindowHolder->SetSensitiveDetector(aSD);

   //------------------------------
   // Beam Line Window (held by Holder, in the center of WH volume)
   //------------------------------
   lengthBeamWindow = 18.0*micrometer;
   G4ThreeVector positionBeamWindow  =
     G4ThreeVector(0,0,-lengthBeamWindow/2);

   G4Tubs* solidBeamWindow = new
     G4Tubs("BeamWindow",0.,(innerWH+1*mm),lengthBeamWindow/2,0.,twopi);

   G4LogicalVolume* logicBeamWindow = new
     G4LogicalVolume(solidBeamWindow, BeamWindowMater,"BeamWindowLV",0,0,0);

   G4VPhysicalVolume* physiBeamWindow = new
     G4PVPlacement(0,positionBeamWindow,"BeamWindow",logicBeamWindow,physiWindowHolder,false,0);

   logicBeamWindow->SetSensitiveDetector(aSD);
   logicBeamWindow->SetVisAttributes(whiteSolid);    
   //   logicBeamWindow->SetVisAttributes(offAtt);    

   //-----------------------------
   // Beam Pipe Flange (one needed here, but define all 4)
   //----------------------------
   G4double innerFlange = 7.5*cm;
   G4double lengthFlange = 1.4*cm;
   G4Tubs *solidFlange = new   // define shape
     G4Tubs("Flange",innerFlange,innerFlange+18.0*micrometer,lengthFlange/2,0.,twopi);
   G4Tubs *solidBigFlange = new   // define shape
     G4Tubs("BigFlange",innerFlange,innerFlange+8*cm,lengthFlange/2,0.,twopi);
   G4LogicalVolume *logicFlange1 = new   // make 4 instances of it
     G4LogicalVolume(solidFlange,Mylar,"Flange1LV",0,0,0);
   G4LogicalVolume *logicFlange2 = new
     G4LogicalVolume(solidFlange,Mylar,"Flange2LV",0,0,0);
   G4LogicalVolume *logicFlange3 = new
     G4LogicalVolume(solidFlange,Mylar,"Flange3LV",0,0,0);
   G4LogicalVolume *logicFlange4 = new
     G4LogicalVolume(solidBigFlange,SSteel,"Flange4LV",0,0,0);

   // make the placement of most downstream one here
   G4ThreeVector positionFlange = G4ThreeVector(0,0,-targetThick/2 -
						thicknessT0 -lengthWH -lengthFlange/2);
   G4VPhysicalVolume *physiFlange1 = new
     G4PVPlacement(0,positionFlange,"Flange",logicFlange1,physiWorld,false,0);

   logicFlange1->SetVisAttributes(blackSolid);
   logicFlange1->SetSensitiveDetector(aSD);
   logicFlange2->SetVisAttributes(blackSolid);
   logicFlange2->SetSensitiveDetector(aSD);
   logicFlange3->SetVisAttributes(blackSolid);
   logicFlange3->SetSensitiveDetector(aSD);
   logicFlange4->SetVisAttributes(blackSolid);
   logicFlange4->SetSensitiveDetector(aSD);

   //-----------------------------
   // Small Beam Pipe (an extension piece) 
   //----------------------------
   G4double radiusSmallPipe = 7.5*cm;
   G4double lengthSmallPipe = 35.*cm;
   G4Material *BeamPipeMater = Aluminium;
   G4Tubs* solidSmallPipe = new
     G4Tubs("SmallPipe",radiusSmallPipe,radiusSmallPipe+3*mm,lengthSmallPipe/2.0,0.,twopi);
   G4LogicalVolume* logicSmallPipe = new
     G4LogicalVolume(solidSmallPipe,BeamPipeMater,"SmallPipeLV",0,0,0);

   G4ThreeVector positionSP = G4ThreeVector(0,0,-targetThick/2 -
					    thicknessT0 - lengthWH -
					    lengthFlange - lengthSmallPipe/2);
   G4VPhysicalVolume *physiSmallPipe = new 
     G4PVPlacement(0,positionSP,"SmallPipe",logicSmallPipe,physiWorld,false,0);

   logicSmallPipe->SetVisAttributes(cyanSolid);
   logicSmallPipe->SetSensitiveDetector(aSD);

   // Another flange
   positionFlange = positionSP + G4ThreeVector(0,0,-lengthSmallPipe/2 -lengthFlange/2);
   G4VPhysicalVolume *physiFlange2 = new
     G4PVPlacement(0,positionFlange,"Flange",logicFlange2,physiWorld,false,0);

   //-----------------------------
   // Beam Pipe Transition Piece (blue on drawing) 
   //----------------------------
   G4double inner2 = 7.5*cm;
   G4double lengthBluePiece = 1.5*cm;
   G4Tubs *solidBluePiece = new   // define shape
     G4Tubs("BluePiece",inner2,inner2+4*cm,lengthBluePiece/2,0.,twopi);
   G4LogicalVolume *logicBluePiece = new   // make 1 instance of it
     G4LogicalVolume(solidBluePiece,SSteel,"BluePieceLV",0,0,0);
   G4ThreeVector positionBluePiece = positionFlange+
     G4ThreeVector(0,0,-lengthFlange/2 -lengthBluePiece/2);
   G4VPhysicalVolume *physiBluePiece = new G4PVPlacement(0,positionBluePiece,"BluePiece",logicBluePiece,physiWorld,false,0);

   logicBluePiece->SetVisAttributes(blueSolid);
   logicBluePiece->SetSensitiveDetector(aSD);

   // Another flange
   positionFlange = positionBluePiece + G4ThreeVector(0,0,-lengthBluePiece/2 -lengthFlange/2);
   G4VPhysicalVolume *physiFlange3 = new
     G4PVPlacement(0,positionFlange,"Flange",logicFlange3,physiWorld,false,0);

   //------------------------------
   // Helium Bag - - "Bag" is Mylar vessel, "HeBag" is the Helium
   // contained in it
   //  ********************* this is a beam pipe extension now! **********
   //------------------------------
   fHeBagLength = HeBagLength-targetThick/2. - thicknessT0/2;
   fHeBagLength = 28. * cm;
   G4ThreeVector positionBag = positionFlange +
     G4ThreeVector(0,0,-lengthFlange/2 - fHeBagLength/2);
   thicknessBag = 100*micrometer;  // resetting these variables to make this
   G4Material* heBagMater = Mylar; // tube into a helium vessel


   G4double inner = 7.5*cm;
   solidBag = new G4Tubs("Bag",0.,inner+thicknessBag,fHeBagLength/2,0.,twopi);
   logicBag = new G4LogicalVolume(solidBag,heBagMater,"BagLV",0,0,0);
   physiBag = new G4PVPlacement(0, positionBag,"Bag",logicBag,physiWorld,false,0);

   /*   G4cout << "mylar bag center: "<<-(fHeBagLength+targetThick)/2<<"mm\n"
 	  << "He volume center: "<<-thicknessBag/2<<" mm from mylar center\n";
   G4cout << "mylar bag is " << fHeBagLength <<" mm overall\n"
 	  << "Helium volume is " << fHeBagLength - thicknessBag <<"mm overall.\n";
   */
   G4ThreeVector positionHe = G4ThreeVector(0,0,-thicknessBag/2);
   G4Tubs* solidHeBag = new G4Tubs("HeBag",0.,inner,fHeBagLength/2 - thicknessBag/2,0.,twopi);
   bagMater = HeliumGas;
   G4LogicalVolume* logicHeBag = new G4LogicalVolume(solidHeBag,bagMater,"HeBagLV",0,0,0);
   G4VPhysicalVolume* physiHeBag = new G4PVPlacement(0, positionHe,"HeliumBag",logicHeBag,physiBag,false,0);

   logicBag->SetSensitiveDetector(aSD);
   logicBag->SetVisAttributes(blueSolid);
   //   logicBag->SetVisAttributes(offAtt);
   logicHeBag->SetSensitiveDetector(aSD);
   logicHeBag->SetVisAttributes(cyanSolid);
   //   logicHeBag->SetVisAttributes(offAtt);

   // Another flange
   positionFlange = positionBag + G4ThreeVector(0,0,-fHeBagLength/2 -lengthFlange/2);
   G4VPhysicalVolume *physiFlange4 = new
     G4PVPlacement(0,positionFlange,"BigFlange",logicFlange4,physiWorld,false,0);
   
   //------------------------------
   // Beam Line (the old model, with a larger radius
   //------------------------------
   G4ThreeVector positionBeamLine = positionFlange + 
     G4ThreeVector(0,0,-lengthFlange/2 - lengthBeamLine/2);
   G4Tubs* solidBeamLine = new
     G4Tubs("BeamLine",0.,radiusBeamLine,lengthBeamLine/2,0.,twopi);
   G4LogicalVolume* logicBeamLine = new
     G4LogicalVolume(solidBeamLine,BeamLineMater,"BeamLineLV",0,0,0);
   G4LogicalVolume* logicBeamLine2 = new
     G4LogicalVolume(solidBeamLine,BeamLineMater,"BeamLine2LV",0,0,0);
   G4VPhysicalVolume* physiBeamLine = new 
     G4PVPlacement(0, positionBeamLine,"BeamLine",logicBeamLine, physiWorld,false,0);
   logicBeamLine->SetVisAttributes(blueFrame);
   //   logicBeamLine->SetVisAttributes(offAtt);
   logicBeamLine->SetSensitiveDetector(aSD);

   // get the distance from end of beam pipe to the target center
   cout <<"\n\n\n\nDistance from center of \"BeamLine\" to target center "
        <<"is "<< positionBeamLine <<endl<<endl<<endl; 
   cout <<"length of BeamLine is = " <<lengthBeamLine/mm <<"mm"<<endl<<endl<<endl;

   //-----------------------------
   // Beam Pipe (a steel or aluminum tube to surround vacuum) 
   //----------------------------
   BeamPipeMater = Aluminium;
   G4double lengthBeamPipe = 1.0*m;
   G4ThreeVector positionBeamPipe = positionBeamLine;
   G4Tubs* solidBeamPipe = new
     G4Tubs("BeamPipe",radiusBeamLine,radiusBeamLine+2*cm,lengthBeamLine/2.0,0.,twopi);
   G4LogicalVolume* logicBeamPipe = new G4LogicalVolume(solidBeamPipe,BeamPipeMater,"BeamPipeLV",0,0,0);
   G4VPhysicalVolume* physiBeamPipe = new G4PVPlacement(0,positionBeamPipe,"BeamPipe",logicBeamPipe, physiWorld, false,0);

   logicBeamPipe->SetVisAttributes(cyanFrame);
   //   logicBeamPipe->SetVisAttributes(offAtt);   
   logicBeamPipe->SetSensitiveDetector(aSD);


   //----------------------------
   // Transition Box - plugin modules for end of the beam pipe into
   // helium bag
   //---------------------------

   G4ThreeVector positionTransBox =  G4ThreeVector(0,0,-(beamToCenter-TBoxZ-lengthBeamWindow -thicknessT0));
   G4Box* solidTransBox = new G4Box("TransBox",TBoxX,TBoxY,TBoxZ);
   G4LogicalVolume* logicTransBox = new G4LogicalVolume(solidTransBox,WireChamberGas,"TransBoxLV");
   //   G4VPhysicalVolume* physiTransBox = new G4PVPlacement(0,positionTransBox, "TransBox",
   //           logicTransBox,physiWorld,false,0);
   logicTransBox->SetSensitiveDetector(aSD);
   logicTransBox->SetVisAttributes(whiteSolid);   
   //   logicTransBox->SetVisAttributes(offAtt);

   /****************** end cap module *****************
   // Helium Bag End Cap (Extension) module
   G4ThreeVector positionEndCapBag = G4ThreeVector(0.,0.,0);
   G4Tubs* solidEndCapBag = new G4Tubs("solidEndCapBag",targetSize,targetSize+thicknessBag,
				       TBoxZ,0,twopi);
   G4LogicalVolume* logicEndCapBag = new G4LogicalVolume(solidEndCapBag,heBagMater,"EndCapBagLV");
   G4VPhysicalVolume* physiEndCapBag = new G4PVPlacement(0,positionEndCapBag, "EndCapBag", 
					 logicEndCapBag, physiTransBox,false,0);
   logicEndCapBag->SetSensitiveDetector(aSD);
   logicEndCapBag->SetVisAttributes(blueSolid);
   //   logicEndCapBag->SetVisAttributes(offAtt);

   //remember that thicknessBag is the FULL thickness of the Helium bag!
   G4ThreeVector positionEndCapHe = G4ThreeVector(0.,0.,thicknessBag);
   G4Tubs* solidEndCapHe = new G4Tubs("solidEndCapHe",0.,targetSize,TBoxZ-thicknessBag/2, 0., twopi);
   G4LogicalVolume* logicEndCapHe = new G4LogicalVolume(solidEndCapHe,bagMater,"EndCapHeLV");
   G4VPhysicalVolume* physiEndCapHe = new G4PVPlacement(0,positionEndCapHe, "EndCapHe",logicEndCapHe,
		   physiTransBox, false,0);

   logicEndCapHe->SetSensitiveDetector(aSD);
   logicEndCapHe->SetVisAttributes(whiteSolid);
   //   logicEndCapHe->SetVisAttributes(offAtt);
   // end Helium bag extension module
   ************************ end cap module ***************/

#if WIRE_CHAMBER_IN_USE

     //Three-Plane Wire Chamber
   // parameters for wire chamber construction: thickness
   //   denotes length along the beam direction Z

   // new wire chamber implemented per PSI drawings
   G4double OuterWindowThickness = 20.*micrometer;  // really aluminized Mylar
   G4double CathodePlaneThickness = 12.*micrometer; // cathodes of aluminized Mylar
   G4double CathodeGap12 = 2.5*mm;


   G4Material* CathodePlaneMater = Mylar; //really aluminized mylar
   G4Material* OuterWindowMater = Mylar; //really aluminized mylar
   G4Material* WireChamberGas = HeliumGas;

   G4double WireDiam = 20.*micrometer;
   G4double HorizWirePitch = 1.*mm;
   G4double VertWirePitch = HorizWirePitch;
   G4Material* WireMater = Tungsten;

   G4ThreeVector positionCathPlane1 = G4ThreeVector(0.,0.,-thicknessBag-(2*CathodeGap12+CathodePlaneThickness/2));
   G4ThreeVector positionCathPlane2 = G4ThreeVector(0.,0.,-thicknessBag);
   G4ThreeVector positionCathPlane3 = G4ThreeVector(0.,0.,-thicknessBag+(2*CathodeGap12+CathodePlaneThickness/2));
   G4ThreeVector positionUpstreamOuterWindow = G4ThreeVector(0.,0.,-thicknessBag-(TBoxZ-OuterWindowThickness/2));
   G4ThreeVector positionDownstreamOuterWindow = G4ThreeVector(0.,0.,-thicknessBag+(TBoxZ-OuterWindowThickness/2));


   //   G4cout <<"cathode plane positions, relative to TBox center:\n";
   //   G4cout << "\t1: "<<-thicknessBag-(2*CathodeGap12+CathodePlaneThickness/2) <<"mm\n"
   //          << "\t2: 0 mm\n\t3: "<< -thicknessBag+(2*CathodeGap12+CathodePlaneThickness/2)<<"mm\n\n";
   //   G4cout << "positionUpstreamOuterWindow = " <<-thicknessBag-(TBoxZ-OuterWindowThickness/2) 
   //          <<"mm\npositionDownstreamOuterWindow = "<< -thicknessBag+(TBoxZ-OuterWindowThickness/2)<<"mm\n\n";

   //Three-Plane Wire Chamber
   G4Box* solidCathPlane1 = new G4Box("CathPlane1",TBoxX,TBoxY,CathodePlaneThickness/2);
   G4LogicalVolume* logicCathPlane1 = new G4LogicalVolume(solidCathPlane1,CathodePlaneMater,"CathPlane1LV");
   //   G4VPhysicalVolume* physiCathPlane1 = new G4PVPlacement(0,positionCathPlane1, "CathPlane1",   logicCathPlane1, physiTransBox,false,0);
   logicCathPlane1->SetSensitiveDetector(aSD);
   
   G4Box* solidCathPlane2 = new G4Box("CathPlane2",TBoxX,TBoxY,CathodePlaneThickness/2);
   G4LogicalVolume* logicCathPlane2 = new G4LogicalVolume(solidCathPlane2,CathodePlaneMater,"CathPlane2LV");
//   G4VPhysicalVolume* physiCathPlane2 = new G4PVPlacement(0,positionCathPlane2, "CathPlane2",  logicCathPlane2, physiTransBox,false,0);
   logicCathPlane2->SetSensitiveDetector(aSD);
   
   G4Box* solidCathPlane3  = new G4Box("CathPlane3",TBoxX,TBoxY,CathodePlaneThickness/2);
   G4LogicalVolume* logicCathPlane3 = new G4LogicalVolume(solidCathPlane3,CathodePlaneMater,"CathPlane3LV");
//   G4VPhysicalVolume* physiCathPlane3 = new G4PVPlacement(0,positionCathPlane3,"CathPlane3",	  logicCathPlane3,physiTransBox,false,0);
   logicCathPlane3->SetSensitiveDetector(aSD);
   
   G4Box* solidOuterWindow = new G4Box("OuterWindow",TBoxX,TBoxY,OuterWindowThickness/2); 

   G4LogicalVolume* logicOuterWindow1 = new G4LogicalVolume(solidOuterWindow,OuterWindowMater,"OuterWindow1LV");
   G4LogicalVolume* logicOuterWindow2 = new G4LogicalVolume(solidOuterWindow,OuterWindowMater,"OuterWindow2LV");
//   G4VPhysicalVolume* physiOuterWindow1 = new G4PVPlacement(0,positionUpstreamOuterWindow,"OuterWindow1",logicOuterWindow1,physiTransBox,false,0);
   logicOuterWindow1->SetSensitiveDetector(aSD);
//   G4VPhysicalVolume* physiOuterWindow2 = new G4PVPlacement(0,positionDownstreamOuterWindow,"OuterWindow2",logicOuterWindow2,physiTransBox,false,0);
   logicOuterWindow2->SetSensitiveDetector(aSD);

   logicCathPlane1->SetVisAttributes(redSolid);
   logicCathPlane2->SetVisAttributes(greenSolid);
   logicCathPlane3->SetVisAttributes(blueSolid);
   logicOuterWindow1->SetVisAttributes(magentaSolid);
   logicOuterWindow2->SetVisAttributes(magentaSolid);
   logicCathPlane1->SetVisAttributes(offAtt);
   logicCathPlane2->SetVisAttributes(offAtt);
   logicCathPlane3->SetVisAttributes(offAtt);
   //   logicOuterWindow->SetVisAttributes(offAtt);
   G4cout << "\n\n\n\n\nlogicCathPlane1 matter = " << CathodePlaneMater << "\n";


   //This WC also needs an "endcap" to close the Helium Bag volume. It is made of a 0.0254mm Mylar sheet.
   G4ThreeVector positionEndCap = G4ThreeVector(0.,0.,TBoxZ-thicknessBag/2);
   G4Tubs* solidEndCap = new G4Tubs("EndCap",0.,targetSize+thicknessBag, thicknessBag/2, 0, twopi);
   G4LogicalVolume* logicEndCap = new G4LogicalVolume(solidEndCap,bagMater,"EndCapLV");
   //   G4VPhysicalVolume* physiEndCap = new G4PVPlacement(0,positionEndCap,"EndCap",logicEndCap,physiTransBox,false,0);
   logicEndCap->SetSensitiveDetector(aSD);
   logicEndCap->SetVisAttributes(magentaSolid);
   logicEndCap->SetVisAttributes(offAtt);


   //----------------------------
   // Wires for the Wire Chamber
   //    Setup and Placement
   //---------------------------
   G4Tubs* solidWireX = new G4Tubs("Wire", 0., WireDiam/2, TBoxX, 0, twopi);
   G4Tubs* solidWireY = new G4Tubs("Wire", 0., WireDiam/2, TBoxY, 0, twopi);

   // -- Horizontal Wires (X direction) --
   G4RotationMatrix* HorizontalRM = new G4RotationMatrix();
   HorizontalRM->rotateY(pi/2.);

   G4int NumHorizWires = (G4int)floor((TBoxY*2/HorizWirePitch) + 0.5);
   G4ThreeVector positionHorizWire = G4ThreeVector(0,-HorizWirePitch*(NumHorizWires-1)/2,
		   -thicknessBag+CathodeGap12);
   //   G4cout <<" positionHorizWire = " <<  positionHorizWire << "\n";

   G4LogicalVolume* logicHorizWire[NumHorizWires];
   G4VPhysicalVolume* physiHorizWire[NumHorizWires];

   for(i = 0; i < NumHorizWires; i++)
     {
       logicHorizWire[i] = new G4LogicalVolume(solidWireX,WireMater,"HorizWireLV",0,0,0);
       //       physiHorizWire[i] = new G4PVPlacement(HorizontalRM,positionHorizWire,"HorizWire", logicHorizWire[i],physiTransBox,false,i);
       positionHorizWire += G4ThreeVector(0.,HorizWirePitch,0.);
       logicHorizWire[i]->SetSensitiveDetector(aSD);
       logicHorizWire[i]->SetVisAttributes(yellowSolid);
       logicHorizWire[i]->SetVisAttributes(offAtt);
     }

   // -- Vertical Wires (Y direction) --
   G4RotationMatrix* VerticalRM = new G4RotationMatrix();
   VerticalRM->rotateX(pi/2.);

   G4int NumVertWires = (G4int)floor(TBoxY*2/VertWirePitch + 0.5);
   G4ThreeVector positionVertWire =  G4ThreeVector(-VertWirePitch*(NumVertWires-1)/2,0.,
				    -(thicknessBag+CathodeGap12));
   //using CathodeGap12 here again to make it symmetric
   //   G4cout <<" positionVertWire = " <<  positionVertWire << "\n"
   //  <<" Horiz wires = " << NumHorizWires << "\n"
   //     <<" Vert wires = " << NumVertWires << "\n";

   G4LogicalVolume* logicVertWire[NumVertWires];
   G4VPhysicalVolume* physiVertWire[NumVertWires];

   for(G4int j = 0; j < NumVertWires; j++)
     {
       logicVertWire[j] = new G4LogicalVolume(solidWireY,WireMater,"VertWireLV",0,0,0);
       //       physiVertWire[j] = new G4PVPlacement(VerticalRM,positionVertWire,"VertWire",    logicVertWire[j],physiTransBox,false,j);
       positionVertWire += G4ThreeVector(VertWirePitch,0.,0.);

       logicVertWire[j]->SetSensitiveDetector(aSD);
       logicVertWire[j]->SetVisAttributes(cyanSolid);
       logicVertWire[j]->SetVisAttributes(offAtt);
     }
   // ----
#endif

   return physiWorld;
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorConstruction::SetTargetMaterial(G4String materialName)
{
   // search the material by its name
   G4Material* pttoMaterial = G4Material::GetMaterial(materialName);
   if (pttoMaterial)
   {
      TargetMater = pttoMaterial;
      logicTarget->SetMaterial(pttoMaterial);
      G4cout << "\n----> The target is " << fTargetLength/cm << " cm of "
      << materialName << G4endl;
   }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorConstruction::SetMagField(G4double fieldValue)
{
   fpMagField->SetFieldValue(fieldValue);
}

// Definition of MulanDetectorConstruction::buildMaterials() has been
// moved to MulanMaterialsConstruction.cc
